<?php

class Onpay_Onpay_Model_PaymentMethod extends Mage_Payment_Model_Method_Abstract
{
    protected $_canCapture             = true;
    protected $_canVoid                = true;
    protected $_canUseForMultishipping = false;
    protected $_canUseInternal         = false;
    protected $_isInitializeNeeded     = true;
    protected $_isGateway               = false;
    protected $_canAuthorize            = false;
    protected $_canCapturePartial       = false;
    protected $_canRefund               = false;
    protected $_canRefundInvoicePartial = false;
    protected $_canUseCheckout          = true;
    
    protected $_code = 'onpay';
    protected $_formBlockType = 'onpay/paymentInformation';
    
    public function getRedirectFormFields()
    {
      $session = Mage::getSingleton('checkout/session');
      
      $order = Mage::getModel('sales/order')->loadByIncrementId($session->getLastRealOrderId());
        
      if (!$order->getId()) {
        return array();
      }
            
      $data['price'] = $this->toFloat($order->getBaseGrandTotal());
      $currency    = $order->getOrderCurrencyCode();
      if ($currency == 'RUB') {
        $currency = 'RUR';
      }
      $data['pay_mode'] = 'fix';
      $data['ticker'] = $currency;
      $data['pay_for'] = $order->getIncrementId();
      
      $data['md5'] = md5($data['pay_mode'].";".$data['price'].";" . $currency . ";" . $data['pay_for'] . ";yes;".$this->getConfig('secret_key'));

      $data['price_final'] = $this->getConfig('price');
      $data['ln'] = $this->getConfig('language');
      $data['f'] = $this->getConfig('form');
      $data['convert'] = $this->getConfig('convert');
      $data['user_email'] = $order->getCustomerEmail();
      $data['url_success_enc'] = urlencode(Mage::getUrl('checkout/onepage/success'));
      $data['url_fail_enc'] = urlencode(Mage::getUrl('checkout/cart'));
     
      return $data;
    }
    
    public function toFloat($sum) {
      $sum = floatval($sum);
      if (strpos($sum, ".")) {
        $sum = round($sum, 2);
      } else {
        $sum = $sum.".0";
      }
      return $sum;
    }
    
    public function getConfig($id) 
    {
      return Mage::getStoreConfig('payment/Onpay/'.$id);
    }
    
    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('onpay/payment/redirect', array('_secure' => true));
    }
    
    public function getOnpayPlaceUrl()
    {
        return 'http://secure.onpay.ru/pay/'.$this->getConfig('login');
    }
    
    public function processNotification()
    {
      $request = $this->oGetData(); 
      
      if ($request['type'] == 'check') {
        $this->oCheck($request);
      } else if($request['type'] == 'pay') {
        $this->oPay($request);
      }
    }
    
    function oPay($request) {
      $_request = $request;
      $pay = array(
        'type' => 'pay',
        'pay_for' => intval($request['pay_for']),
        'payment.amount' => $this->toFloat($request['payment']['amount']),
        'payment.currency' => trim($request['payment']['way']),
        'amount' => $this->toFloat($request['balance']['amount']),
        'currency' => trim($request['balance']['way']),
        'key' => $this->getConfig('secret_key'),
      );
      $pay['signature_string'] = implode(";", $pay);
      $pay['signature'] = sha1($pay['signature_string']);
      $payOut = array(
        'type' => 'pay',
        'status' => 'false',
        'pay_for' => intval($request['pay_for']),
        'key' => $this->getConfig('secret_key'),
      );
      
      if($this->oValidate($request, $pay['signature'])) {
        $pay_for = $request['pay_for'];
        $transaction_id = $request['payment']['id'];
        
        $order = Mage::getModel('sales/order');
        $order->loadByIncrementId($pay_for);
        
        if (!$order->getId()) {
          Mage::throwException(Mage::helper('onpay')->__('Cannot load order'));
        }
        
        if($order->getStatus() == 'pending') {
          if ($order->canInvoice()) {
            $order->getPayment()->setTransactionId($transaction_id);
            $invoice = $order->prepareInvoice();
            $invoice->register()->pay();
            
            Mage::getModel('core/resource_transaction')
                          ->addObject($invoice)
                          ->addObject($invoice->getOrder())
                          ->save();

            $order->setState(
                          Mage_Sales_Model_Order::STATE_PROCESSING, true,
                          $message,
                          $notified = true
                      );
            $payOut['status'] = 'true';         
          }
        }
      }
      $this->oResponse($payOut, $request);
    }
    
    public function oCheck($request) {
      $check = array(
        'type' => 'check',
        'pay_for' => intval($request['pay_for']),
        'amount' => $this->toFloat($request['amount']),
        'currency' => trim($request['way']),
        'mode' => trim($request['mode']),
        'key' => $this->getConfig('secret_key'),
      );
      $check['signature_string'] = implode(";", $check);
      $check['signature'] = sha1($check['signature_string']);
      $checkOut = array(
        'type' => 'check',
        'status' => 'false',
        'pay_for' => intval($request['pay_for']),
        'key' => $this->getConfig('secret_key'),
      );
      $amount = floatval($request['amount']);
      if($this->oValidate($request, $check['signature'])) {
        $checkOut['status'] = 'true';
      }
      $this->oResponse($checkOut, $request);
    }
    
    public function oResponse($response, $request) {
      $response['signature_string'] = implode(";", $response);
      $response['signature'] = sha1($response['signature_string']);
      $out = "{\"status\":{$response['status']},\"pay_for\":\"{$response['pay_for']}\",\"signature\":\"{$response['signature']}\"}";
      
      header("Content-type: application/json; charset=utf-8");
      echo iconv("cp1251", "utf-8", $out);
      die;
    }
    
    public function oValidate($request, $signature) {
      $request['pay_for'] = intval($request['pay_for']);
      if($request['signature'] != $signature) {
        return false;
      }
      return true;
    }
    
    public function oGetData() {
      $ret = false;
      if(function_exists('json_decode')) {
        if(isset($GLOBALS['__inputData'])) {
          $ret = $GLOBALS['__inputData'];
        } elseif($hSource = fopen('php://input', 'r')) {
          $input = "";
          while (!feof($hSource)) {
            $input .= fread($hSource, 1024);
          }
          fclose($hSource);
          $input = trim($input);

          $ret = json_decode($input, true);
          if(is_null($ret)) $ret = json_decode(iconv("cp1251", "utf-8", $input), true);

          $GLOBALS['__inputData'] = $ret;
        }
      }
      return $ret;
    }
    
    public function initialize($paymentAction, $stateObject)
    {
        $state = Mage_Sales_Model_Order::STATE_NEW;
        $stateObject->setState($state);
        $stateObject->setStatus(Mage::getSingleton('sales/order_config')->getStateDefaultStatus($state));
        $stateObject->setIsNotified(false);
        return $this;
    }
}